;Tim Carleton
;1/29/11
;updated 03/26/11 to subtract whole transit rather than just half
; This program reads in a file with the light curve of a transit, and
; then determines weather or not the transit is symetrical

;Calling procedure:
; .r transitsymetry.pro
;subtransit=transitsymetry('transit.txt',0.0)
;The first argument is the name of the file with the light curve. This
;file is assumed to be a 2 column file with the first column being
;times and the second column being the relative flux of the light curve
;(assumed to be normalized to 1).
;
;The second argument is the time of midtransit
;
;The function returns a two dimensional array of the subtraction of
;the flipped transit from the normal transit
;
;Keywords:
;nointerpolation: set this keyword to 1 if you do not want this
;procedure interpolating between points in the transit for the
;analysis. It is recomended that you use this keyword only if there
;are large holes in important parts of the transit, as the
;interpolation routine works best for most transits.
;starttime: The time used as the start of the transit. If this keyword
;is not supplied, the transit start time is assumed to be the last
;point in the initial baseline where the normalized flux is greater
;than (or equal to) 1.
;endtime:The time used as the end of the transit. If this keyword
;is not supplied, the transit end time is assumed to be the first
;point in the final baseline where the normalized flux is greater
;than (or equal to) 1.
;The start and end times are only used to calculate the rms of the
;non-transit flux, so it is not imperative that they be exact, so
;except for especially noisy transits, these keywords are not
;necessary.
;epsilon: a confidence interval used if the nointerpolation keyword is
;used. If the nointerpolation keyword is set to 1, then the program
;compares flux values of similar times. Set this keyword to the
;difference in times that is acceptable for a flux comparison. The
;default is 5% of the total light curve time.
;sigmadetection: How many sigma to report a deviation from a symetric
;transit. Default is 3.
;waittime: The number of seconds to show a plot. A value of -1 does no
;plotting. The default is 3.
;Example call with keywords:
;subtransit=transitsymetry('transit.txt',0.0,starttime=-.45,endtime=.46,/nointerpolation,epsilon=.005,simgadetection=5)

;The program works by subtracting the flipped transit from the
;original transit (from now on: the subtracted
;transit). On a perfectly symetric transit, the subtracted transit
;should all be 0. It then calculates the standard deviation of the
;non-transit flux. It then reports any points that lay sigmadetection
;standard deviations away from a value of 0 in the subtracted
;transit. The subtracted transit is set to the return value.

;The program will plot certain things along the way to show progress.
;The first plot is the transit exactly as it is in the transit file.
;The second plot is the flipped transit interpolated to the points on
;the original trasit times. THIS WILL NOT SHOW UP IF NOINTERPOLATION=1.
;The final plot is the subtracted transit. Any points that are sigma
;standard deviations away from 0 are shown as diamonds.

FUNCTION transitsymetry, lightcurvefilename,midtransittime,nointerpolation=ni,starttime=st,endtime=et,epsilon=epsilon,sigmadetection=sg,waittime=wt

iplt = 1				; 1=hardcopy


;If (iplt eq 1) then begin
  set_plot, 'ps'
  device, filename='Asymmetry_Plot.ps',/color	;set name of hardcopy
;endif 


;If the light curve file does not exist, stop the program
if~(file_test(lightcurvefilename)) then begin
  print,'No file of name: "'+lightcurvefilename+'" detected'
  return,0
endif
openr,lun,lightcurvefilename,/get_lun
n=0
;This loop is used to figure out how many points are in the transit file
while ~ EOF(lun) do begin
    readf,lun,var
    n=n+1
endwhile
;Based on the number of points calculated in the previous loop,
;construct a 2xn array to put the transit in.
transita=fltarr(2,n)
close,lun
;close,/all
free_lun,lun
openr,lun,lightcurvefilename,/get_lun
readf,lun,transita
close,lun
;close,/all
free_lun,lun
;We only use the first two columns of the transit for our analyis
transit=fltarr(2,n)
transit[0,*]=transita[0,*]
transit[1,*]=transita[1,*]
;It is important that the transit is monotonically increasing in time,
;so we sort the transit
s=sort(transit[0,*])
transit[0,*]=transit[0,s]
transit[1,*]=transit[1,s]
if(n_elements(wt) eq 0) then wt=3
;Here we plot the raw transit data
if(wt ne -1) then begin
    plot,transit[0,*],transit[1,*],psym=3,/ynoz
    wait,wt
endif
time=reform(transit[0,*])
makesymetric,time,midtransittime,loc=symetrictransitloc
flux=reform(transit[1,symetrictransitloc])
transit=fltarr(2,n_elements(symetrictransitloc))
flippedtransit=fltarr(2,n_elements(transit[1,*]))
transit[0,*]=time-midtransittime
transit[1,*]=flux
flippedtransit[0,*]=-1*reverse(reform(transit[0,*]))
flippedtransit[1,*]=reverse(flux)
min=max([min(transit[0,*]),min(flippedtransit[0,*])])
max=min([max(transit[0,*]),max(flippedtransit[0,*])])
wforward=where(transit[0,*] ge min and transit[0,*] le max)
transit2=fltarr(2,n_elements(wforward))
transit2[0,*]=transit[0,wforward]
transit2[1,*]=transit[1,wforward]
wback=where(flippedtransit[0,*] ge min and flippedtransit[0,*] le max)
flippedtransit2=fltarr(2,n_elements(wback))
flippedtransit2[0,*]=flippedtransit[0,wback]
flippedtransit2[1,*]=flippedtransit[1,wback]
transit=transit2
flippedtransit=flippedtransit2
if(wt ne -1) then begin
    plot,transit[0,*],transit[1,*],psym=3,/ynoz
    oplot,flippedtransit[0,*],flippedtransit[1,*],psym=4,symsize=.76
    wait,wt
endif
;now we interpolate the times on the flipped transit to the times on the
;original transit
if(~keyword_set(ni)) then begin
    flippedinterpol=interpol(flippedtransit[1,*],flippedtransit[0,*],transit[0,*])
    flippedinterpol=[transit[0,*],flippedinterpol]
;Plot the interpolated points
    if(wt ne -1) then begin
        plot,transit[0,*],transit[1,*],psym=3,/ynoz
        oplot,flippedinterpol[0,*],flippedinterpol[1,*],psym=4,symsize=.76
        wait,wt
    endif
;Here we assign the subtracted transit to the original transit-the
;interpolated flipped transit
    subtracted=fltarr(2,n_elements(reform(transit[0,*])))
    subtracted[1,*]=transit[1,*]-flippedinterpol[1,*]
    subtracted[0,*]=transit[0,*]
;If we are told to not interpolate: we have to loop through all the
;points on the original transit and compare them to similar points
;on the flipped transit
endif else begin
;Set the default epsilon to 5% of the total time of the transit
    if(n_elements(epsilon) eq 0) then $
      epsilon=0.05*(transit[0,n_elements(transit[0,*])-1]-transit[0,0])
    subtracted=transit
    subtracted[1,*]=0
    for i=0,n_elements(transit[0,*])-1 do begin
;Loop along the transit, and then the subtracted flux is the flux on
;the orginal transit minus the flux on the flipped transit at the
;closest time
        sametime=min(abs(transit[0,*]-flippedtransit[0,i]),loc)
;However, only do the comparison if the points are within time epsilon
;of each other
        if(sametime-flippedtransit[0,i] ge epsilon) then $
          subtracted[1,i]=0 $
        else $
          subtracted[1,i]=transit[1,i]-flippedtransit[1,loc]
    endfor
endelse
;Plot the subtracted transit
if(wt ne -1) then $
  plot,subtracted[0,*],subtracted[1,*]

;Now we calculate the scatter in the transit baseline to report points
;that are statistically significant in the subtracted transit

;Here we find the left baseline of the transit
left=where(transit[0,*] lt 0)
if(n_elements(st) eq 0) then begin
    lb=where(transit[1,left] ge 1, countl)
    if(lb[0] ne -1) then $
      leftbase=transit[*,0:lb[countl-1]]
    st=transit[0,lb[0]]
endif else begin
    stimes=min(abs(transit[0,*]-st),lb)
    leftbase=transit[*,0:lb]
endelse
;Here we find the right baseline of the tranist
right=where(transit[0,*] gt 0)
if(n_elements(et) eq 0) then begin
    rb=where(transit[1,right] ge 1,countr)
    if(rb[0] ne -1) then $
      rightbase=transit[*,rb[0]:*]
    et=transit[0,rb[0]]
endif else begin
    endtimes=min(abs(transit[0,*]-et),rb)
    rightbase=transit[*,rb : *]
endelse
;Here we find the scatter of the baseline
sigmabase=stddev([reform(leftbase[1,*]),reform(rightbase[1,*])])
;print, 'Standard Deviation of baseline: ',sigmabase
if(n_elements(sg) eq 0) then $
  sg=3
;Report any points that are more than sg simga away from a completely
;symetric transit (a completely symetric transit is if the subtracted
;transit is all 0)
w=where(abs(subtracted[1,*]) gt sg*sigmabase)
if(w[0] ne -1) then begin
    print, 'Asymetry at times: '
    print,transit[0,w]*(-1)
;There may be points that are a certain sigma away from the symetric
;transit, but just artifacts of a noisy transit, so it is important to
;look at a graph of where the anti-symetric points are to see if there
;is a real asymetric feature.
    if(wt ne -1) then $
      oplot,subtracted[0,w],subtracted[1,w],psym=4
endif

;If (iplt eq 1) then begin
   device, /close
  ; set_plot,'x'
;endif
;Finally, return the subtracted transit.
return,subtracted

;openw,lun,'test.dat',/get_lun 
;size2 =  n_elements(subtracted[0,*]) 
;sub = fltarr(2,size2)
;sub[0,*] = subtracted[0,*]
;sub[1,*] = subtracted[1,*] 
;printf,lun,sub
;close, /all 

end
