; Name: Turner_model_fit_monte.pro (UAEXOMOP- University of Arizona EXOplanet MOdeling Package)
; This program inputs a light curve of an exoplanet and fits a model fit to it.  
; This program performs a Levenberg-Marquardt least-squares fit to the data to find the best fit model
; Finds the best model fit using the Mandel & Agol (2002) prescription. 
; This program uses the Bootstrap Monte Carlo technique to find errors on the transit parameters.  
; Note: This program accounts for red noise using the Prayer Bead Method (Residual-Permutation Method) (eg. Fernandez et al. 2009)
;       This method has been updated to use the error bars from the residuals. 
;
;WHAT IT DOES: 
;	1.)  Find a linear fit to your out of transit baseline and fits a linear fit. 
;             Then it divides your transit by the linear fit to get a de-trended light curve. 
;	2.) Fits your de-trended transit data using Levenberg-Marquardt least-squares fit to a Mandel & Agol (2002) model
;       3.)  Finds robust errors using the Bootstrap Monte Carlo technique
;       4.)  Finds the asymmetry of the transit about the calculated mid-transit time
;	5.)  Uses the Prayer Bead Method (Residual-Permutation Method) to access the influence of red noise on the transit and calculates new errors
;
; INPUTS: 
;        EXTERNAL FILE: transit.dat (with error bars)
;        WILL ASK: ~Ingress Point, ~Egress Point, Period,
;                    initial parameter guesses for (Rp/R*,a/R*,Inclination,Mid-Transit Time, 
;                                                   Linear and Quadratic Limb Darkening),
;                     # of Monte Carlo Simulations
;
; OUTPUTS: 
;  Final_parameters_prayer.dat - The fitted parameters of the planet with error bars derived from the Prayer Bead Red Noise method
;                                 (rp/R*,a/R*,Inclination, Mid-Transit Time, Linear and Quadratic LD) 
;	  Final_parameters.dat - a. The fitted parameters of the planet with error bars from the Levenberg-Marquardt Monte Carlo method
;                                    (rp/R*,a/R*,Inclination, Mid-Transit Time, Linear and Quadratic Limb Darkening) 
;			          b. Duration of transit in days and mins (with errors)
;			          c. Cadence of observations (days and secs)
;			          d. Out of Transit RMS 
;			          e. RMS of residuals 
;           			  f. Reduced Chi-Squared from best fit-model
;			          g. Lowest Monte Carlo Reduced Chi-Squared 
;			          h. STDEV of the transit subtracted by the mirror image of itself (from transitsymmetry.pro)
;				  i. Baseline Intercept calculated from Detrend_linfit.pro
;                                 j. Baseline slope calculated from Detrend_linfit.pro
;          Best-Fit.dat        - Time, Phase, original transit, error, best-fit model, and residuals
;          Input_Value.dat     - A file containing the input values and what values were fixed. 
;          plot.ps 	       - Plot of the Best-fit model, data, and residuals	
;          monte_value2.dat    - Each Monte-Carlo run (Best-fit model-Monte Carlo value)  
;          monte_value.dat     - Each Monte-Carlo run (Monte Carlo values and Chi-Squared)
;          guass.ps 	       - Plot of the distribution of fitted parameters from the Monte Carlo simulations
;          Asymmetry_Plot.ps   - 4 Plots showing the asymmetry process (the 4th plot shows the subtraction) 
;          symmetry_test       - time and flux used for the asymmetry test 
;      monte_value_prayer.dat  - Each Monte Carlo run from the Prayer Bead Method (Best-fit model-monte carlo value)  
;          guass_prayer.ps     - Plot of the distribution of fitted parameters by the Prayer Bead Method from the Monte Carlo simulations 
;
; LIMITATIONS:
;            - Does not fit for period 
;            - Only works for eccentricity = 0 and omega = 0 
;
; ENSURING YOU HAVE RELIABLE  ERROR BARS: 
;           	- Check to make sure that the guass.ps and gauss_prayer.ps plots are gaussian. If they are not, run your model with more/less fixed parameters 
;		- Always choose the largest error bars from the Final_parameters.dat and Final_parameters_prayer.dat files. 
;		  If there is significant red noise in your images the Final_parameters_prayer.dat error bars will be bigger. 
; 		- To get your final error bars multiple your errors bars in final_parameters.dat or Final_parameters_prayer.dat by sqrt(Chi-Squared), 
;                 where Chi-Squared is the Reduced Chi-Squared in Final_parameters.dat file. Only do this if the Reduced Chi-Squared is greater than one. 
;
; MODIFICATION HISTORY 
; 2013/02/13   Jake Turner (LPL) V1.0 
; 	       Created
; 2013/02/17   Jake Turner (LPL) V2.0
;              Added linear detrending of Out of transit baseline 

run_prayer   = 1		; Red noise fit (1=Yes) & (0=NO)                                (Default=1)
run_symmetry = 1		; Symmetry calculation (Transit - Mirror image of itself)       (Default=1)
run_baseline = 1		; Run a linear fit of the baseline and divide it out of transit (Default=1)

;------------------------------------------------
;-----READ USER INPUTS --------------------------
;------------------------------------------------
transitfile = ' '
foldername = ' '
read ,'Enter Folder Name (ex. 28mar12_run1): ',foldername
file_mkdir, foldername
read, 'Enter Transit File: ',transitfile
read, 'At what data point does your ingress end?: ', ingress
read, 'At what data point does your egress start?: ', egress
read, 'Is your time in phase? (0=YES,1=NO): ', phase_indicator
read, 'Initial Guess for Rp/R*: ', rp
read, 'Would you like to fit Rp/R* (0=YES,1=NO): ', parinfo_rp
read, 'Initial Guess for a/R*: ', a_rstar
read, 'Would you like to fit a/R* (0=YES,1=NO): ',parinfo_ar
read, 'Initial Guess for Inclination (degrees): ',incl 
read, 'Would you like to fit Inclination (0=YES,1=NO) ' ,parinfo_inc
read, 'Initial Guess for Mid-Transit Time: ', mid_Transit
read, 'Would you like to fit the Mid-Transit Time (0=YES,1=NO) ' ,parinfo_mid
read, 'Linear Limb Darkening Coefficient: ', linlimb
read, 'Would you like to fit Linear Limb Darkening Coefficient (0=YES,1=NO) ' ,parinfo_lin
read, 'Quadratic Limb Darkening Coefficient: ', quadlimb
read, 'Would you like to fit Quadratic Limb Darkening Coefficient (0=YES,1=NO) ' ,parinfo_quad
read, 'Orbital Period (days): ', period
read, 'Number of Monte Carlo Simulation to Run (10000 is a good number): ', N 

openw,9,'Input_Values.dat'
printf,9,'Input Values into Model'
printf,9,'************************************'
printf,9, 'Transit File:',transitfile,FORMAT='(A13,2x,A20)'
printf,9,'Time in Phase? (0=YES,1=NO)', phase_indicator
printf,9, 'Rp/R*:',rp,FORMAT='(A6,9x,F9.7)
printf,9, 'a/R*:',a_rstar,FORMAT='(A5,10x,F9.6)
printf,9, 'Inclination:',incl,FORMAT='(A12,1x,A20)'
printf,9, 'Mid-Transit Time:   ', mid_Transit
printf,9, 'Period:', period,FORMAT='(A7,19x,F10.8)'
printf,9, 'Linear Limb Darkening Coefficient:       ', linlimb
printf,9, 'Quadratic Limb Darkening Coefficient     ', quadlimb
printf,9, 'Number of Monte Carlo Simulation to Run: ', N
printf,9,'Fit for Rp/R*? (0=YES,1=NO)', parinfo_rp 
printf,9,'Fit for a/R*?  (0=YES,1=NO)', parinfo_ar 
printf,9,'Fit for I?     (0=YES,1=NO)', parinfo_inc 
printf,9,'Fit for Tmid?  (0=YES,1=NO)', parinfo_mid
printf,9,'Fit for Linear?(0=YES,1=NO)', parinfo_lin
printf,9,'Fit for Quad?  (0=YES,1=NO)', parinfo_quad
printf,9,'Number of Ingress Pts', round(ingress)
printf,9,'Number of Egress Pts', round(egress)
close,9 
file_move,['Input_Values.dat'],foldername


;------------------------------------------------
;-----Number of Monte Carlo Simulations----------
;------------------------------------------------
N = N  ; Monte Carlo Simulations (make sure it is > than 1000)
	    ; 10000 takes about 10 minutes to run 
	    ; 100000 takes about 50 minutes to run
;------------------------------------------------
;-----READ TRANSIT FILE--------------------------
;------------------------------------------------

readcol,transitfile,time,flux_transit,error
size = n_elements(time) 

;save original transit
flux_transit_or = flux_transit

If run_baseline eq 1 then begin
;----------------------------------------------------------------------------------------------------
;----------------------------------De-trend Baseline of Transit--------------------------------------
;----------------------------------------------------------------------------------------------------

egress = round(egress)
ingress = round(ingress)

time_base 		= fltarr(size)
flux_transit_base 	= fltarr(size)
error_base		= fltarr(size)
count = 0 

for i=0,size-1 do begin
  If ((i lt ingress) OR (i gt egress)) then begin 
      time_base[i] 		= time[i] 
      flux_transit_base[i]	= flux_transit[i]
      error_base[i] 		= error[i]	
      count = count + 1
  endif
endfor
print,"count",count
time_base2 		= fltarr(count)
flux_transit_base2 	= fltarr(count)
error_base2		= fltarr(count)

time_base2[0:ingress-1]         	 = time_base[0:ingress-1]
time_base2[ingress:count-1] 	 	 = time_base[egress+1:size-1]
flux_transit_base2[0:ingress-1] 	 = flux_transit_base[0:ingress-1]
flux_transit_base2[ingress:count-1]	 = flux_transit_base[egress+1:size-1]
error_base2[0:ingress-1]	  	 = error_base[0:ingress-1]
error_base2[ingress:count-1] 		 = error_base[egress+1:size-1]

Detrend = Detrend_linfit(time_base2,flux_transit_base2,error_base2)		;run detrender

for i=0,size-1 do begin
	flux_transit[i] = flux_transit_or[i] / (Detrend[0] + Detrend[1]*time[i]	)	;detrend
endfor

;renormalize

base = fltarr(count) 
base[0:ingress-1] = flux_transit[0:ingress-1]
base[ingress:count-1] = flux_transit[egress+1:size-1]

average = MEAN(base)
flux_transit = flux_transit/average		;update

endif 						;end baseline fit

;------------------------------------------------
;-----initialize parameters----------------------
;------------------------------------------------
rp_monte 	    	= fltarr(N)
rp_monte2	    	= fltarr(N)
a_rstar_monte		= fltarr(N)
a_rstar_monte2		= fltarr(N)
incl_monte    		= fltarr(N)
incl_monte2    		= fltarr(N)
mid_monte   		= fltarr(N)
mid_monte2   		= fltarr(N)
lin_monte2		= fltarr(N)
quad_monte2		= fltarr(N)
lin_monte		= fltarr(N)
quad_monte		= fltarr(N)
fit_start 		= fltarr(7) 
flux_transit2 		= fltarr(size)
;chi_squared		= fltarr(N)
chi			= fltarr(N)

;------------------------------------------------
;-----Output Files------------------------------
;------------------------------------------------
openW,1,'monte_value.dat',width=250
openw,12,'monte_value2.dat'
openw,5,'Best-Fit.dat' , width=250


;------------------------------------------------
;-----Setup of Initial Guess----------------------
;------------------------------------------------
incl = (!PI/180)*incl	;Convert to radians !IMPORTANT

fit_start[0] 	= rp
fit_start[1] 	= a_rstar
fit_start[2] 	= incl
fit_start[3] 	= mid_transit 
fit_start[4] 	= linlimb
fit_start[5]    = quadlimb
fit_start[6] 	= period

;Update Time if it is in phase 
If phase_indicator eq 0 then begin
time = time*period		;only do if you input time is in phase
endif


;------------------------------------------------
;-----Create parameter info structure------------
;------------------------------------------------
parinfo = replicate({fixed:0,MPSIDE:0,limited:[0,0], limits:[0.D,0.D]},7)	

;set limits on Rp/R* 
parinfo[0].limited(0) = 1
parinfo[0].limits(0)  = 0.001
parinfo[0].limited(1) = 1
parinfo[0].limits(1)  = 0.4

;set limits on a/R* 
parinfo[1].limited(0) = 1
parinfo[1].limits(0)  = 1.5

;set limits for Linear Limb Darkening 
parinfo[4].limited(0) = 1
parinfo[4].limits(0) = 0
parinfo[4].limited(1) = 1
parinfo[4].limits(1) = 1

;set limits for Quadratic Limb Darkening 
parinfo[5].limited(0) = 1
parinfo[5].limits(0) = -1
parinfo[5].limited(1) = 1
parinfo[5].limits(1) = 1


parinfo[0].fixed = byte(parinfo_rp)		; Update Fit for Rp/R*
parinfo[1].fixed = byte(parinfo_ar)		; Update Fit for a/R*
parinfo[2].fixed = byte(parinfo_inc)		; Update Fit for Inclination
parinfo[3].fixed = byte(parinfo_mid)		; Update Fit for Mid-Transit
parinfo[4].fixed = byte(parinfo_lin)		; Update Fit for linear limb darkening
parinfo[5].fixed = byte(parinfo_quad)		; Update Fit for Quad limb darkening
parinfo[6].fixed = 1			 	; Update Fit for Period (Never Fit!)

for i=0,6 do begin 		;set parameter structure 
  parinfo[i].mpside = 2		;  2 - two-sided derivative (f(x+h) - f(x-h))/(2*h) 
endfor 				; the sidedness of the finite difference when computing numerical derivatives. 

						
;---------------------------------------------------------------------------------------------------------
;--------------------------------------------Find Best Fit (Initial Guess)--------------------------------
;---------------------------------------------------------------------------------------------------------

	flux_model = model_fit(time,fit_start)
	FitValue = mpfitfun('model_fit',time,flux_transit,error,fit_start,PERROR=perror,PARINFO=parinfo,/AUTODERIVATIVE,/quiet,BESTNORM= chi_squared2,DOF=DOF)
	Best_fit = FitValue
	best_model = model_fit(time,Best_fit) 

;find Chi_squared
	print, chi_squared2/DOF
	limit = chi_squared2/DOF

;repeat to get an initial guess that produces a reduced_chi_squared lt 1
repeat begin 
	flux_model = model_fit(time,fit_start)
	FitValue = mpfitfun('model_fit',time,flux_transit,error,fit_start,PERROR=perror,PARINFO=parinfo,/AUTODERIVATIVE,/quiet,BESTNORM= chi_squared2,DOF=DOF)
	Best_fit = FitValue
	best_model = model_fit(time,Best_fit) 

;find Chi_squared
	print, chi_squared2/DOF
	fit_start[0] = fit_start[0] + 0.001		; small change to recalibrate LM fit 

endrep until chi_squared2/DOF lt limit			; If Reduced Chi_squared is lt 1 stop					
	
;------------------------------
;--Reduced Chi-Squared---------
;------------------------------
chi_reduced = chi_squared2/DOF	 ; chi/(DOF)



	openw,8,'Initial_parameters'
	printf,8,Best_fit[0], PERROR[0]
	printf,8,Best_fit[1], PERROR[1]
	printf,8,Best_fit[2]*(180/!PI), PERROR[2]*(180/!PI)	; convert to Degrees
	printf,8,Best_fit[3], PERROR[3]
	printf,8,Best_fit[4], PERROR[4]
	printf,8,Best_fit[5], PERROR[5]
	close,8
	file_move,['Initial_parameters'],foldername

	fit_start[0] 	= rp			;reset fit_start[0] 

;--------------------------------------
;----Output Best Fit Model--------------
;--------------------------------------

best_model = model_fit(time,Best_fit) 

;------------------------------------------------
;-----Calculate Residuals and Phase--------------
;------------------------------------------------

res = fltarr(1,size) 
res = flux_transit - best_model

phase = ( time - FitValue[3] ) / period

;-----------------------------------------------
;-----Print Transit and Best fit Model----------
;-----------------------------------------------

print_best = fltarr(7,size)
print_best[0,*] = time 
print_best[1,*] = phase
print_best[2,*] = flux_transit_or
print_best[3,*] = flux_transit
print_best[4,*] = error
print_best[5,*] = best_model
print_best[6,*] = res
printf,5,print_best
close,5
file_move,['Best-Fit.dat'],foldername

;--------------------------------------
;-----Plot Best Fit Model--------------
;-------------------------------------
set_plot,'ps'
device,filename='plot.ps',/color
color_array=['navy blue','red','orange','green','blue','purple', 'white', 'yellow','black']
ploterror, phase, flux_transit,error,yrange=[MIN(best_model)-14*STDEV(res),1.01],psym=SYM(1),symsize=0.5,color=Fsc_Color(color_array[8]),xtitle='Time since mid transit (Days)',ytitle='Relative Flux'
;oploterror, time,flux_transit, error
oplot, phase, best_model,color=Fsc_Color(color_array[1]),thick = 8
oplot,phase,res*0+MIN(best_model)-10*STDEV(res),color=Fsc_Color(color_array[1]),thick=8
oplot,phase,res+(MIN(best_model)-10*STDEV(res)),color=Fsc_Color(color_array[8]),psym=SYM(1),symsize=.5
device,/close
file_move,['plot.ps'],foldername


;------------------------------------------------------------------------------------------------------------------
;-------------Perform the Bootstrap Monte Carlo (find best value and error)----------------------------------------
;------------------------------------------------------------------------------------------------------------------
print, '*******************************************************************************' 
print, '*****************************MONTE CARLO STARTING******************************'
print, '*******************************************************************************' 


for i=0L, N-1 do begin			; begin Monte Carlo iterations
	If i eq 5000 then begin 
		print, 'Are We There Yet (You are at 5000)' 
	endif
	If i eq 9000 then begin 
	   print, 'Just Keep Swimming (You are at 9000)'
	endif

	random = RANDOMN(seed, size)	;find random number with a mean of 0 and stdev of 1
					; important (seed needs to change every iteration), set by time on computer. 
      for j=0, size-1 do begin
		flux_transit2(j)=flux_transit(j)+random(j)*error(j)	
      endfor

	FitValue = mpfitfun('model_fit',time,flux_transit2,error,fit_start,PERROR=perror,PARINFO=parinfo,yfit=yfit,/quiet,/AUTODERIVATIVE,BESTNORM= chi_squared,DOF=DOF)
	
;------------------------------
;----Find Chi-Squared----------
;------------------------------
	
;reduced Chi-Squared
chi[i] = chi_squared/DOF

	;find differences between assumed best fit value
	rp_monte2[i] 		= Best_fit[0]-FitValue[0]
	a_rstar_monte2[i] 	= Best_fit[1]-FitValue[1]
	incl_monte2[i]    	= Best_fit[2]-FitValue[2]
	mid_monte2[i]   	= Best_fit[3]-FitValue[3]
	lin_monte2[i] 		= Best_fit[4]-FitValue[4] 
	quad_monte2[i] 		= Best_fit[5]-FitValue[5]

	rp_monte[i] 		= FitValue[0]
	a_rstar_monte[i] 	= FitValue[1]
	incl_monte[i]    	= FitValue[2]
	mid_monte[i]   		= FitValue[3]
	lin_monte[i] 		= FitValue[4]
	quad_monte[i] 		= FitValue[5]	
	printf,12,rp_monte2[i],a_rstar_monte2[i],incl_monte2[i], mid_monte2[i],lin_monte[i],quad_monte[i]
	printF,1,rp_monte[i],a_rstar_monte[i],incl_monte[i], mid_monte[i],lin_monte[i],quad_monte[i],chi[i]

Endfor 											;End MC simulations 
	close,1
	close,12
	file_move,['monte_value.dat','monte_value2.dat'],foldername

;----------------------------------------------------------------------------------------
;------Fit Gaussian to Distribution And find SIGMA (Error) 
;----------------------------------------------------------------------------------------
set_plot,'ps'
device,filename='guass.ps',/color
openw,2,'Final_Parameters.dat'


;------------------------------
;------Fit Gaussians-----------
;------------------------------

If parinfo_rp eq 0 then begin 
	histogauss, rp_monte2, A 		; Fit a gaussian function to the distribution 
	printf,2,Best_fit[0], A[2],'Rp/R*',FORMAT='(F9.7,2X,F9.7,2x,A5)' 	
endif

If parinfo_ar eq 0 then begin
	histogauss, a_rstar_monte2, B 
	printf,2,Best_fit[1], B[2],'a/R*',FORMAT='(F9.7,2X,F9.7,2x,A4)'	
endif

If parinfo_inc eq 0 then begin
	histogauss, incl_monte2, C 
	;convert inclination to degrees 
	printf,2,Best_fit[2]*(180/!PI), C[2]*(180/!PI), 'Inclination (degrees)',FORMAT='(F9.6,2X,F9.7,2x,A21)'	
endif

If parinfo_mid eq 0 then begin
	histogauss, mid_monte2, D
	printf,2,Best_fit[3], D[2],'Mid-Transit (days)',FORMAT='(F10.7,2X,F9.7,2x,A18)'	
endif 

If parinfo_lin eq 0 then begin
	histogauss, lin_monte, E
	printf,2,Best_fit[4], E[2],'Linear Limb',FORMAT='(F9.7,2X,F9.7,2x,A15)'	
endif 

If parinfo_quad eq 0 then begin
	histogauss, quad_monte, F
	printf,2,Best_fit[5], F[2],'Quadratic Limb',FORMAT='(F9.7,2X,F9.7,2x,A15)'	
endif 
	

;------------------------------------------------------------------------------------
;------Output Useful Values (OoT Baseline, Cadence, Duration)------------------------
;------------------------------------------------------------------------------------
;file_compile,'Useful_values.pro',PRO_NAME=USEFUL_VALUES     ;compile Useful_Values.pro

Value = Useful_Values(time,res,best_model,flux_transit,size)
printf,2,Value[1],Value[2],' Duration (Days)',FORMAT='(F9.7,2X,F9.7,1x,A16)'  
printf,2,Value[1]*24*60,Value[2]*24*60,' Duration (Min)',FORMAT='(F9.5,2X,F9.7,1x,A15)' 
printf,2,Value[3],' Cadence (days)',FORMAT='(F9.7,12X,A15)'
printf,2,Value[3]*24*60*60, ' Cadence (secs)' ,FORMAT='(F9.5,3X,A24)'
printf,2,Value[0],'OoT STDEV',FORMAT='(F9.7,13x,A9)'
printf,2,STDEV(res)*1000,' RMS of Residuals (mmag)',FORMAT='(F9.7,12X,A24)'
printf,2,chi_reduced,'Reduced Chi-Squared',FORMAT='(F9.5,12X,A19)'
printf,2,MIN(chi),' Lowest MC Reduced Chi^2',FORMAT='(F9.5,12X,A24)'
printf,2, Detrend[0],' Baseline Intercept', FORMAT='(F9.5,13X,A19)'
printf,2, Detrend[1],' Baseline Slope' , FORMAT='(F9.5,13x,A15)'

;close everything
;If run_symmetry eq 0 then begin 
;close,/all
;endif


If run_symmetry eq 1 then begin
;------------------------------------------------------------------------------------------------------------------
;----------------------------Calculation the of Symmetry of the Transit--------------------------------------------
;----------This part of the program uses transitsymetry.pro and makesymetric.pro written by Tim Carleton (03/26/11)
;------------------------------------------------------------------------------------------------------------------

openw,56,'symmetry_test'
print_transit = fltarr(2,size) 
print_transit[0,*] = time 
print_transit[1,*] = flux_transit
printf,56,print_transit
close,56

mid = Best_fit[3]
subtransit = transitsymetry('symmetry_test',mid)

printf,2,stdev(subtransit[1,*])*1000,'STDEV of Asymmetry (mmag)',FORMAT='(F9.7,13X,A25)'

endif

close,/all
;device, /close
file_move,['guass.ps','symmetry_test','Final_Parameters.dat','Asymmetry_Plot.ps'],foldername

If run_prayer eq 1 then begin

;-------------------------------------------------------------------------------------------------------
;--------------------------------------(Red Nose Estimation)--------------------------------------------
;----------------------------Prayer Bead Method (Residual-Permutation Method)-------------------------
;-------Used to Find out how much correlated noise (Red noise) is a problem-----------------------------
;-----The general idea is to perform the Bootstrap MC and add the residuals (with errors to the transit) 
;-----See Southworth 2008 and Fernandez et al. 2009 for more information on PB method-------------------
;------------------------------------------------------------------------------------------------------
;----This routine does exactly what is described in these references with the exception that we --------
;----include the error from the residuals and add residual+evenly disturbed error to the transit, ------ 
;----thus allowing for more than d shifts (d is number of data points in the transit)--------------------
;-----We run this routine N*(size/size-1) times (N is # of Monte Carlo Sims, size # of data points) -----
;--------------------------------------------------------------------------------------------------------

print,'******************************************************************************' 
print, '*****************************PRAYER BEAD STARTED*****************************'
print,'******************************************************************************' 

;------------------------------------------------
;-----Calculate Residuals for Prayer Bead method
;------------------------------------------------

prayer_number = N/size
prayer_number = round(prayer_number) 

res_new			= fltarr(size*(prayer_number+1))

for j=0,prayer_number do begin
for i = 0,size-1 do begin
	res_new(i+size*j)     = res(i)
endfor
endfor


runs = size*(prayer_number-1) 


;------------------------------------------------------------------------------------------------------------------
;-------------Find Error Through Monte Carlo-----------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------
openw,10,'monte_value_prayer.dat'
openw,20,'Final_Parameters_prayer.dat'

for i=0L, runs-1 do begin				; begin Monte Carlo iterations
	If i eq 5000 then begin 
		print, 'Where are we? (You are at 5000)' 
	endif

	
	random = RANDOMN(seed, size,/UNIFORM)	;find random number with a mean of 0 and 1 (evenly)
						; important (seed needs to change every iteration), set by time on computer. 

      for j=0, size-1 do begin
		error_term = 2*(random(j) - 0.5) 				; make between -1 and 1 
		flux_transit2(j)=flux_transit(j)+ res_new(j+i)*error_term
		;flux_transit2(j)=flux_transit(j)+ res_new(j+i)

      endfor

	FitValue = mpfitfun('model_fit',time,flux_transit2,error,fit_start,PERROR=perror,PARINFO=parinfo,yfit=yfit,/quiet,/AUTODERIVATIVE)
	
	;find differences between assumed best fit value
	rp_monte[i] 		= Best_fit[0]-FitValue[0]
	a_rstar_monte[i] 	= Best_fit[1]-FitValue[1]
	incl_monte[i]    	= Best_fit[2]-FitValue[2]
	mid_monte[i]   		= Best_fit[3]-FitValue[3]
	lin_monte[i] 		= Best_fit[4]-FitValue[4] 
	quad_monte[i] 		= Best_fit[5]-FitValue[5]
	printf,10,rp_monte[i],a_rstar_monte[i],incl_monte[i], mid_monte[i],lin_monte[i],quad_monte[i]
Endfor 


;---------------------------------------------------------
;------Fit Gaussian to Distribution And find SIGMA (Error) 
;---------------------------------------------------------
set_plot,'ps'
device,filename='guass_prayer.ps',/color


If parinfo_rp eq 0 then begin 
	histogauss, rp_monte, A 							; Fit a gaussian function to the distribution 
	printf,20,Best_fit[0], A[2],'Rp/R*',FORMAT='(F9.7,2X,F9.7,2x,A5)' 	
endif

If parinfo_ar eq 0 then begin
	histogauss, a_rstar_monte, B 
	printf,20,Best_fit[1], B[2],'a/R*',FORMAT='(F9.7,2X,F9.7,2x,A4)'	
endif

If parinfo_inc eq 0 then begin
	histogauss, incl_monte, C 

	;convert inclination to degrees 
	Best_fit[2] = Best_fit[2]*(180/!PI)
	C[2]	    = C[2]*(180/!PI)
	printf,20,Best_fit[2], C[2], 'Inclination (degrees)',FORMAT='(F9.6,2X,F9.7,2x,A21)'	
endif

If parinfo_mid eq 0 then begin
	histogauss, mid_monte, D
	printf,20,Best_fit[3], D[2],'Mid-Transit (days)',FORMAT='(F10.7,2X,F9.7,2x,A18)'	
endif 

If parinfo_lin eq 0 then begin
	histogauss, lin_monte, E
	printf,20,Best_fit[4], E[2],'Linear Limb',FORMAT='(F9.7,2X,F9.7,2x,A15)'	
endif 

If parinfo_quad eq 0 then begin
	histogauss, quad_monte, F
	printf,20,Best_fit[5], F[2],'Quadratic Limb',FORMAT='(F9.7,2X,F9.7,2x,A15)'	
endif 

printf,20,runs,'Number of Simulations',FORMAT='(I5,18X,A21)'



close,/all
file_move,['Final_Parameters_prayer.dat','guass_prayer.ps','monte_value_prayer.dat'],foldername

endif 


save,/ALL,FILENAME ='everything.sav'	;save everything
file_move,['everything.sav'],foldername ;move
FILE_COPY, [transitfile], foldername	;copy transit file


;To restore
;restore,FILENAME='everything.sav'

print,'The End' 


END
