;NAME: 
;	beta
;
;PURPOSE:
; This program finds the affect of red noise using the Time-Averaging Technique 
; Aka the Beta method (Pont et al 2006, 373,231)
; Uses the definition of beta (Equation 10) in in Carter & Winn 2009 (APJ, 704, 51)
;
; To be used in EXOMOP but can be used separately. 
;
; INPUT: 
;        time (HJD) 
;        res  (residuals of data-model)
;	 error
;
;OUTPUT: 
;       Beta_value.dat
;       beta_plot.ps
;       Sigma_white_noise
;       Sigma_red_noise
;       Beta (as defined by Carter&Winn2009)
;	error in white noise 
; 	error in red noise
;
;REQUIREMENTS: 
;              bin.pro 
;	       beta_model.pro (Equation 9 from Pont et al 2006)
;
;
;MODIFICATION HISTORY
;
; 3/11/13 --- Jake Turner (LPL) 2013 
; 		program created
;

function beta,time,res,error
	
size = n_elements(time)
cadence_diff = fltarr(size-1)
for i=0,size-2 do begin 
   cadence_diff[i] = abs(time[i+1]-time[i]) 
endfor 
   cadence = MAX(cadence_diff) 			;find max cadence (in case of overlapping transits)

size = n_elements(time) 

numbins = size/5		;size/5 
bin_size = fltarr(1,numbins) 
bin_time = fltarr(1,numbins) 

bin_rms  = fltarr(1,numbins) 
bin_fun  = fltarr(1,numbins) 
print_holder 	  = fltarr(3, numbins) 


;-----------------------------------------------------------------------------------------
;--------------Find Bin STDEV and Bin-Size -------------------------------------------------
;-----------------------------------------------------------------------------------------
openw,1,'Beta_value.dat'

for i=1,numbins-1 do begin
       bin_time[0]    = cadence*2
       bin_size[0]    = 1
If n_elements(res) gt 1 then begin
       bin_rms[0]     = STDEV(res)
endif
	;bin time
   	bin_time[i]    = cadence +bin_time[i-1]   	; 30secs*i
   	bin_input      = bin_time[i]

	;bin transit
If n_elements(res) gt 1 then begin
	binned         = bin(time,res,error,bin_input) 
endif 	
	;rms of binned residuals
	rms_stuff      = binned[1,*]

	If  n_elements(rms_stuff) gt 1 then begin 
  		bin_rms[i]     =  STDEV(rms_stuff)
		print_holder[1,i] = bin_rms[i] 
		print_holder[1,0] = bin_rms[0]
	endif
	
	;bin size
   	avg_size       = MEAN(binned[4,*])
   	bin_size[i]    = round(avg_size) 		; number of points!!!!
	print_holder[0,i] = bin_size[i] 
	print_holder[0,0] = bin_size[0] 

	;sigma/sqrt(N)
        bin_fun[0] = (bin_rms[0]) 
        bin_fun[i] = (bin_rms[0]/sqrt(bin_size[i]))
	print_holder[2,0] = bin_fun[0]
 	print_holder[2,i] = bin_fun[i]

endfor

printf,1,print_holder
close,1

set_plot,'ps'
device,filename='beta_plot.ps',/color
color_array=['navy blue','red','orange','green','blue','purple', 'white', 'yellow','black']
plot,bin_size,print_holder[1,*],psym=SYM(1),symsize=2,/YLOG,/XLOG,color=Fsc_Color(color_array[8]),xtitle='N',Ytitle='Relative RMS',xrange=[min(bin_size),max(bin_size)],yrange=[min(bin_rms),max(bin_rms)]
oplot,bin_size,bin_fun,color=Fsc_Color(color_array[1]),thick = 4
device,/close

;------------------------------------------------
;-----Create parameter info structure------------
;------------------------------------------------
parinfo = replicate({fixed:0,MPSIDE:0,limited:[0,0], limits:[0.D,0.D]},2)	


;set limits on white_noise
parinfo[0].limited(0) = 1
parinfo[0].limits(0)  = 0
parinfo[0].limited(1) = 1
parinfo[0].limits(1)  = 2

;set limits on red_noise
parinfo[1].limited(0) = 1
parinfo[1].limits(0)  = 0
parinfo[1].limited(1) = 1
parinfo[1].limits(1)  = 2


for i=0,1 do begin 		;set parameter structure 
  parinfo[i].mpside = 2		;  2 - two-sided derivative (f(x+h) - f(x-h))/(2*h) 
endfor 				; the sidedness of the finite difference when computing numerical derivatives. 

fit_start = fltarr(2)
fit_start[0] = 0.001
fit_start[1] = 0.001

bin_error = 0.000045	;set generic error to run mpfitfun
FitValue = mpfitfun('beta_model',bin_size,bin_rms,bin_error,fit_start,PERROR=perror,PARINFO=parinfo,/AUTODERIVATIVE,BESTNORM= chi_squared_beta,DOF=DOF)

Best_fit = FitValue

;Equation 10 in Carter & Winn 2009 (APJ, 704, 51)
Beta = sqrt( 1 + (Best_fit[1]/Best_fit[0])^2 )

beta_stuff = fltarr(5,1) 
beta_stuff[0,0] = Best_fit[0]   	; white noise
beta_stuff[1,0] = Best_fit[1]   	; red noise
beta_stuff[2,0] = PERROR[0]     	; error in white noise
beta_stuff[3,0] = PERROR[1]     	; error in red noise
beta_stuff[4,0] = Beta 			; Beta

print, 'Beta',Beta
If Beta eq 1 then begin 
	print,'BAMMMMMMMMMMMM!!!!!!!'
	print,'WOOOOOOOTTTTTTTT!!!!!' 
	print,'You have very little RED Noise in your transit!! :)'
endif


return,beta_stuff

end
