; Name: 
;       BIN
; PURPOSE: 
; 	  Bin a set of inputted data to the bin size inputed
; 	  Outputs both the weighted mean and the median value of the data points binned
;
; To be used in red_beta.pro but can be used separately. 
;
; INPUT: 
;        time (HJD) 
;        normalized_flux
;	 error
;	 bin size (days)
;
;OUPUT:  Binned time (HJD)
;	 Binned Flux (weighed mean) 
;	 Binned Flux (median)
; 	 Binned Error (variance of the weighted mean) 
;        Number of images binned
;
;CALLING SEQUENCE:
;		  binned = bin(time,data,error,bin_input) 
;
;MODIFICATION HISTORY
;
; 3/11/13 --- Jake Turner (LPL) 2013 
; 		program created
;

function bin,time,flux_transit,error,bin_size

;-----------------------------------------------------------------------------------------
;		Set Up Bin Boundaries
;-----------------------------------------------------------------------------------------

size = n_elements(time) 	;find size of input 

; total time of light curve
total_time= time(size-1) - time(0) 	

;number of bins
maxbin = round(total_time/bin_size)	 
If maxbin le 1 then begin 
maxbin = 2
endif

;set up variables 
bin_time = fltarr(1,maxbin)
bin_bound = fltarr(1,maxbin) 
lower_bound = time(0)

;step size 
bins = (time(size-1)-time(0))/maxbin

;find bin Boundaries 
for i=0,maxbin-1 do  begin 
      bin_bound[i] =  lower_bound + bins*i
endfor


;set up time variables
for i=0,maxbin-2 do begin 
    bin_time[i] = ( bin_bound[i+1] + bin_bound[i] ) /2
endfor


;-----------------------------------------------------------------------------------------
;		     Find Number of Points in each bin
;-----------------------------------------------------------------------------------------

;set variables
errorflux = fltarr(1,size-1)
binflux = fltarr(1,size-1)
binflux_med = fltarr(1,size-1)

point_bin = fltarr(1,maxbin-1)


For jj=0,maxbin-2 do begin	;for loop for bin range

 N = 0		; set counter to zero	 

	For ii=0,size-1 do begin 		;for loop for each entry in file
		If( ( time[ii] ge bin_bound[jj] ) and (time[ii] lt bin_bound[jj+1] ) ) then begin 	  
	           N = N + 1	
		endif 
	endfor
 point_bin[jj] = N	;save number of points in bin

endfor	

;-----------------------------------------------------------------------------------------
;		     Find Bin Values
;-----------------------------------------------------------------------------------------

holder = 0 

For jj=0,maxbin-2 do begin				;for loop for bin range

      sum = 0  ;set to zero 
      sumx = 0

      N = point_bin[jj] 	;set N as the number of points in each bin
     
	If N gt 0 then begin 
           holder = fltarr(1,N)
    	   holder_error = fltarr(1,N)
           weight = fltarr(1,N)

	  iii = 0 				;reset counter for holder
		For ii=0,size-1 do begin 	;for loop for each entry in file
			If( ( time[ii] ge bin_bound[jj] ) and (time[ii] lt bin_bound[jj+1] ) ) then begin
                               	If(iii lt N) then begin
					holder[iii]       = flux_transit[ii] 		;individual data points
                                	holder_error[iii] = error[ii]			;individual errors
                                        weight[iii] = 1.0/holder_error[iii]^2		;weight for each value
   					sum    = weight[iii] + sum			;weight sum
  					sumx   = weight[iii]*holder[iii] + sumx		;weight*value sum
				endif
				iii = iii + 1
			
			EndIf
		Endfor
	EndIf	


;------------------------------
;----Calculate weighted mean---
;------------------------------ 

   xmean  	  = sumx/sum				;Weighted Average
   error_flux 	  = 1.0/sum				;1/sum(weights)
   binflux[jj]    = xmean				;output weighted average
   errorflux[jj]= sqrt(error_flux)			;output weighted mean
If n_elements(holder) gt 1 then begin 
   binflux_med[jj]  = MEDIAN(holder,/EVEN)		;output median value
endif

endfor		;end for loop over bins


;------------------------------
;---Output Bin Values----------
;------------------------------
final_bin      = fltarr(5,maxbin-1)		;create Output
final_bin[0,*] =    bin_time[0:maxbin-2] 	;weighted time
final_bin[1,*] =     binflux[0:maxbin-2]	;weighted mean
final_bin[2,*] = binflux_med[0:maxbin-2]	;median
final_bin[3,*] =   errorflux[0:maxbin-2]	;weighted error
final_bin[4,*] =   point_bin[0:maxbin-2]  	;number of points averaged per bin


return,final_bin

end
