;+
; NAME:
;   EXOMOP_mcmc
;
; PURPOSE: 
; 	Make independent Markov Chain Monte Carlo chains to fit data and return the parameter distributions.
;	Uses a Differential Evolution Markov Chain Monte Carlo fit (ter Braak, 2006)
;       http://www.stat.columbia.edu/~gelman/stuff_for_blog/cajo.pdf. 
;
;    The program stops when the chains are well-mixed, as defined by
;   Ford 2006 (http://adsabs.harvard.edu/abs/2006ApJ...642..505F)
;   using the Gelman-Rubin Statistic and number of independent draws,
;   or when each chain has taken MAXSTEPS, whichever is first.
;
;  Uses exofast_demc and exofast_plotdist created by Eastman 2013
;
; CALLING SEQUENCE:
;    exomop_mcmc,bestpars,parinfo,maxsteps[,tofit=tofit]
;
; INPUTS:
;	BESTPARS   - Array of the best-fit parameters from the Bootstrap Monte Carlo 
;                    Levenberg-Marquardt fit from EXOMOP. An accurate initial
;                    fit is required to find the correct step size. 
;	PARINFO   - Array of the parameters to fit
;       MAXSTEPS  - Maximum number of steps to take in the MCMC
;
;OUTPUTS:
; 	Plot_MCMC.ps 		  - Plot of the best-fit model from the MCMC 
;	Final_Parameters_MCMC.dat - The Final fit parameters and associated errors 
;	tofit 			  - Parameter indexes fit in the MCMC 
;
;SUBROUTINES CALLED (in /sub)
;	EXOFAST_DEMC.pro, EXOFAST_PLOTDIST.pro, exofast_random.pro, exomop_chi2 
;
; COMMON BLOCKS:
;   CHI2_BLOCK - 
;   COMMON chi2_block,transit
;
; MODIFICATION HISTORY
; 2013/03/23     Jake Turner (LPL) 
;		 Created 
 
pro exomop_mcmc,bestpars,parinfo,maxsteps,nchains,tofit=tofit,res_mcmc=res_mcmc,best_model_mcmc=best_model_mcmc,bestpars_mcmc=bestpars_mcmc,mcmcerrors= mcmcerrors


;Function that minimizes the Chi^2 (Mandel & Agol 2002 code) 
chi2func = 'exomop_chi2'

randomfunc = 'exofast_random'
;common block
COMMON chi2_block,transit

nthin = 1L

determinant = 1d0
npars = 9 

chi2 = 0.d0

;; exclude parameters as desired
tofit = indgen(npars)
;fit rp and mid 

for i=0,8 do begin 
  If parinfo[i] eq 1 then begin
	tofit = tofit[where(tofit ne i)] 
  endif 
endfor 

;tofit = tofit[where(tofit ne 1 and tofit ne 2 and tofit ne 4 and tofit ne 5 and tofit ne 6 and tofit ne 7 and tofit ne 8 and tofit ne 9)]

namendx = tofit		;set name labels
;; output labels 
latexnames =[['R_{P}/R_{*}',  'Radius of planet in stellar radii'],$      
              ['a/R_{*}',     'Semi-major axis in stellar radii'],$        
              ['I',           'Inclination'],$                      
	      ['T_{mid}',     'Time of Mid-transit (\bjdtdb)'],$               
              ['u_1',         'linear limb-darkening coeff'],$             
              ['u_2',         'quadratic limb-darkening coeff'],$           
              ['P',           'Period (days)'],$                           
              ['e',           'Eccentricity'],$                            
              ['\omega_*',    'Argument of periastron (degrees)']]      
namendx = tofit
latexnames = latexnames[*,namendx]
parnames = TeXtoIDL(latexnames[0,*],font=0)


;; must calculate scale before MCMC, because of scatter in Enoch relations
scale = exofast_getmcmcscale(bestpars,chi2func,tofit=tofit)

exofast_demc, bestpars, chi2func, pars, chi2=chi2, tofit=tofit,$
                 scale=scale, nthin=nthin,maxsteps=maxsteps,nchains=nchains,$
                 burnndx=burnndx, seed=seed, randomfunc=randomfunc, /REMOVEBURN

;; combine all chains
   sz = size(pars)
   npars = sz[1]
   nsteps = sz[2]
   nchains = sz[3]
   pars = reform(pars,npars,nsteps*nchains)
   chi2 = reform(chi2,nsteps*nchains)
   minchi2 = min(chi2,bestndx)


  ;; make PDFs, covriances plots
	exofast_plotdist, pars, finalpars, $
                  angular=angular,$
                  covarname=covarfile, parnames=parnames,PDFNAME='probability_plot.ps',/degrees

bestpars_mcmc = bestpars				;set to best fit with LM

;override with MCMC values
for i=0,n_elements(tofit)-1 do begin 
	If tofit[i] eq 0 then begin 
		bestpars_mcmc[0]  = finalpars[0,i] 	;rp/R*
	endif 
	If tofit[i] eq 1 then begin 
		bestpars_mcmc[1]  = finalpars[0,i] 	;a/R*
	endif 
	If tofit[i] eq 2 then begin	
		bestpars_mcmc[2]  = finalpars[0,i] 	;inclination
	endif
	If tofit[i] eq 3 then begin
		bestpars_mcmc[3]  = finalpars[0,i] 	;mid-transit
	endif
	If tofit[i] eq 4 then begin
		bestpars_mcmc[4]  = finalpars[0,i] 	;lin limb 
	endif
	If tofit[i] eq 5 then begin
		bestpars_mcmc[5]  = finalpars[0,i] 	;quad limb 
	endif
endfor

;Set Errors of values
mcmcerrors = fltarr(2,9)
for i=0,n_elements(tofit)-1 do begin 
	If tofit[i] eq 0 then begin 
		mcmcerrors[0,0]  = finalpars[1,i] 	;rp/R*
		mcmcerrors[1,0]  = finalpars[2,i] 	;rp/R*
	endif 
	If tofit[i] eq 1 then begin 
		mcmcerrors[0,1]  = finalpars[1,i] 	;a/R*
		mcmcerrors[1,1]  = finalpars[2,i] 	;a/R*
	endif 
	If tofit[i] eq 2 then begin	
		mcmcerrors[0,2]  = finalpars[1,i] 	;inclination
		mcmcerrors[1,2]  = finalpars[2,i] 	;in

	endif
	If tofit[i] eq 3 then begin
		mcmcerrors[0,3]  = finalpars[1,i] 	;mid-transit
		mcmcerrors[1,3]  = finalpars[2,i] 
	endif
	If tofit[i] eq 4 then begin
		mcmcerrors[0,4]  = finalpars[1,i] 	;lin limb 
		mcmcerrors[1,4]  = finalpars[2,i] 
	endif
	If tofit[i] eq 5 then begin
		mcmcerrors[0,5]  = finalpars[1,i] 	;quad limb 
		mcmcerrors[1,5]  = finalpars[2,i] 
	endif
endfor


;bestpars_mcmc[0]  = finalpars[0,0] 			;override with MCMC values 
;bestpars_mcmc[3]  = finalpars[0,1]			;override with MCMC values 
best_model_mcmc = model_fit(transit.time,bestpars_mcmc) 

res_mcmc = transit.flux - best_model_mcmc

set_plot,'ps'
device,filename='plot_mcmc.ps',/color
color_array=['navy blue','red','orange','green','blue','purple', 'white', 'yellow','black']
ploterror, transit.time, transit.flux,transit.err,yrange=[MIN(best_model_mcmc)-14*STDEV(res_mcmc),1.01],psym=SYM(1),symsize=0.5,color=Fsc_Color(color_array[8]),xtitle='Time ',ytitle='Relative Flux'
oplot, transit.time, best_model_mcmc,color=Fsc_Color(color_array[1]),thick = 8
oplot, transit.time,res_mcmc*0+MIN(best_model_mcmc)-10*STDEV(res_mcmc),color=Fsc_Color(color_array[1]),thick=8
oplot, transit.time,res_mcmc+(MIN(best_model_mcmc)-10*STDEV(res_mcmc)),color=Fsc_Color(color_array[8]),psym=SYM(1),symsize=.5
device,/close

finalpars_mcmc = fltarr(3,9)
bestpars_mcmc[2]    = bestpars_mcmc[2]*(180/!PI)	;convert to degrees
mcmcerrors[0,2]     = mcmcerrors[0,2]*(180/!PI)
mcmcerrors[1,2]     = mcmcerrors[1,2]*(180/!PI)

finalpars_mcmc[0,*] = bestpars_mcmc
finalpars_mcmc[1,*] = mcmcerrors[0,*]
finalpars_mcmc[2,*] = mcmcerrors[1,*]

openw,lun,'Final_Parameters_MCMC.dat',/get_lun
printf,lun, finalpars_mcmc
close,lun


end

